/*
 * Simulator of MCS51 (uc.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>

#include "uccl.h"
#include "hwcl.h"
#include "memcl.h"


/*
 * Abstract ucrocontroller
 */

cl_uc::cl_uc(void):
  cl_base()
{
  mem= new cl_list(MEM_TYPES, 1);
  hws= new cl_list(1, 1);
}


cl_uc::~cl_uc(void)
{
  delete mem;
  delete hws;
}


int
cl_uc::init(void)
{
  cl_base::init();
  mk_rom();
  mk_xram();
  mk_iram();
  mk_sfr();
  mk_hw();
  return(0);
}


/* Making variables */

int
cl_uc::mk_rom(void)
{
  class cl_mem *m= new cl_mem(MEM_ROM, 0x1000);
  mem->add_at(MEM_ROM, m);
  return(m != 0);
}

int
cl_uc::mk_xram(void)
{
  class cl_mem *m= new cl_mem(MEM_XRAM, 0x1000);
  mem->add_at(MEM_XRAM, m);
  return(m != 0);
}

int
cl_uc::mk_iram(void)
{
  class cl_mem *m= new cl_mem(MEM_IRAM, 0x1000);
  mem->add_at(MEM_IRAM, m);
  return(m != 0);
}

int
cl_uc::mk_sfr(void)
{
  class cl_mem *m= new cl_mem(MEM_SFR, 0x1000);
  mem->add_at(MEM_SFR, m);
  return(m != 0);
}

int
cl_uc::mk_hw(void)
{
  return(0);
}


/* Read/write simulated memory */

ulong
cl_uc::read_mem(enum mem_class type, long addr)
{
  class cl_mem *m;

  if (m= (class cl_mem*)mem->at(type))
    return(m->read(addr));
  return(0);
}

void
cl_uc::write_mem(enum mem_class type, long addr, ulong val)
{
  class cl_mem *m;

  if (m= (class cl_mem*)mem->at(type))
    {
      m->write(addr, &val);
      //m->mem[addr]= val;
    }
}


/* Register callback hw objects for mem read/write */

void
cl_uc::register_hw_read(enum mem_class type, long addr, class cl_hw *hw)
{
  class cl_mem *m;
  class cl_memloc *l;

  if (m= (class cl_mem*)mem->at(type))
    {
      if ((l= m->read_locs->get_loc(addr)) == 0)
	{
	  l= new cl_memloc(addr);
	  l->init();
	  m->read_locs->add(l);
	}
      l->hws->add(hw);
    }
}

void
cl_uc::register_hw_write(enum mem_class type, long addr, class cl_hw *hw)
{
}


/* End of uc.cc */
